<?php
namespace Odhen\API\Service;

use \SimpleXMLElement;

class PromocoesInt {
	protected $sessionHandler;
	
	public function __construct(
		\Helpers\Environment $sessionHandler
	){
        $this->sessionHandler = $sessionHandler;
	}
    public function chamadaAPIPromocoesInt($rota, $params){
        self::LogPromoInt('');
        self::LogPromoInt('chamadaAPIPromocoesInt');
        $URL = $this->sessionHandler->getUserInfo()['CDURLINTEPROMO'];
        $URL = rtrim($URL, '/') . "/" . $rota;
        $curl = curl_init();
        self::LogPromoInt('URL: '.$URL);
		curl_setopt($curl, CURLOPT_URL, $URL);
		curl_setopt($curl, CURLOPT_CUSTOMREQUEST, 'POST');
		curl_setopt($curl, CURLOPT_HTTPHEADER, array('Content-Type:application/xml'));

        $XMLparam = self::ArrayToXml($params);
        self::LogPromoInt('PARAMETERS: '. preg_replace( '/[\n,  ]/', "", $XMLparam ));
        curl_setopt($curl, CURLOPT_POSTFIELDS,$XMLparam);

        curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, false);
        curl_setopt($curl, CURLOPT_ENCODING,'');
        curl_setopt($curl, CURLOPT_MAXREDIRS,10);
        curl_setopt($curl, CURLOPT_FOLLOWLOCATION, true);
        curl_setopt($curl, CURLOPT_HTTP_VERSION , CURL_HTTP_VERSION_1_1);

        curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($curl, CURLOPT_TIMEOUT, 20);
        curl_setopt($curl, CURLOPT_CONNECTTIMEOUT, 20);

        $curl_response = curl_exec($curl);
        $err    = curl_errno($curl);
        $errmsg = curl_error($curl);
        $httpcode = curl_getinfo($curl, CURLINFO_HTTP_CODE);
        curl_close($curl);
        self::LogPromoInt('HTTPCODE: '.$httpcode);
        if ($httpcode ==='404' || $httpcode === 404){
            self::LogPromoInt('Erro 404');
            throw new \Exception("<br>Erro 404. Servidor não encontrado. Verifique se a URL parametrizada esta correta.");
        } else {
            self::LogPromoInt('RESPONSE: '.$curl_response);
        }
        if ($err){
            self::LogPromoInt('errmsg: '.$errmsg);
            if ($err === 28){
                if(empty($errmsg)){
                    $errmsg = "<br>N&atilde;o foi poss&iacute;vel estabelecer uma conex&atilde;o com o servidor.";
                }
                throw new \Exception($errmsg);
            } else {
                self::LogPromoInt('err: '.$err);
                throw new \Exception("<br>Ocorreu um erro na chamada com o servidor:<br><br>" . $errmsg);
            }
        }
        if (empty($curl_response)){
            self::LogPromoInt('empty curl_response');
            throw new \Exception("<br>N&atilde;o houve retorno do servidor.");
        }
        try{
            $result = self::XmlToJson($curl_response);
            $result = json_decode($result, true);
        } catch(\Exception $e){
            $result = json_decode($curl_response, true); 
            if($result){
                if(isset($result['error'])){
                    $result['success'] = false;    
                }
            }
        }
		if (empty($result)) {
            $result = array('success'=>false,'message'=>$curl_response);
        }
		if (!isset($result['success'])){
			$result['success']=false;
		}
		$result['success'] = ($result['success'] === true || $result['success'] === 'true');
		if($result['success']){
            self::LogPromoInt('SUCCESS true');
            self::LogPromoInt('');
			return $result;
		} else {
            self::LogPromoInt('SUCCESS false');
            self::LogPromoInt('');
			if(isset($result['message'])){
				throw new \Exception('Erro: '.$result['message']);
			} else {
				throw new \Exception("Ocorreu um erro não esperado na consulta ao servidor.".(!empty($rota)?' Endpoint '.$rota.'.':''));
			}
		}        
	}

    public function XmlToJson ($xml) {
        self::LogPromoInt('XmlToJson');
        $xml = str_replace(array("\n", "\r", "\t"), '', $xml);
        $xml = trim(str_replace('"', "'", $xml));
        $simpleXml = simplexml_load_string($xml);
        $json = json_encode($simpleXml);
        self::LogPromoInt($json);
        return $json;
    }

    function ArrayToXml($array, $xml = null){
        $boprimeira = false;
        if($xml == null){
            $xml = new SimpleXMLElement('<root/>');
            $boprimeira = true;
            self::LogPromoInt('ArrayToXml');
        }        
        foreach ($array as $key => $value) {
            if(is_int($key) && is_array($value)){
                self::ArrayToXml($value, $xml);
            } else {
                if(is_int($key)){
                    if($key===0){
                        $key = '0';
                    } else {
                        $key = strval($key);
                    }
                }
                if(is_bool($key)){
                    if($key){
                        $key = 'true';
                    }else {
                        $key = 'false';
                    }
                }
                if(is_array($value)){
                    $label = $xml->addChild($key);
                    self::ArrayToXml($value, $label);
                }
                else {
                    $xml->addChild($key, $value);
                }
            }
        }
        if($boprimeira){
            //self::LogPromoInt($xml->asXml());
        }
        return $xml->asXml();
    }

    public function LogPromoInt($texto){
        return self::Log('PromoInt',$texto);
    }

    public function Log($tipo,$texto){
        if($tipo=='PromoInt'){
            $tipo ='PromocoesInt';
        }
        $paths = ['LOG/', $tipo.'/', (new \DateTime('NOW', new \DateTimeZone('America/Bahia')))->format('m-Y')];
        $logPath = dirname(__DIR__) . '/../../../../../../../';
        foreach ($paths as $path){
            $logPath = $logPath . $path;
            if (!is_dir($logPath)){
                mkdir($logPath, 0700);
            }
        }
        $logPath = $logPath .'/'. (new \DateTime('NOW', new \DateTimeZone('America/Bahia')))->format('d-m-Y') . '.txt';
        $horaAtual = (new \DateTime('NOW', new \DateTimeZone('America/Bahia')))->format('d/m/Y H:i:s');
        file_put_contents($logPath, $horaAtual . ' - ' . $texto . "\r\n", FILE_APPEND);
    }
}